"""MADSci Terminal User Interface.

This module provides an interactive terminal UI for managing and
monitoring MADSci labs.
"""

from madsci.client.cli.tui.app import MadsciApp

__all__ = ["MadsciApp"]
